//*****************************************************
// BK1088 & RadioLCD control progtam
// Bk1088/I2C subroutines
//              By Minarai Shokunin
//    http://crystal-radio.blogzine.jp/blog/
//*****************************************************

#include "P24FJ64GA004.h" 
#include "prototypes.h" 

// Externals

extern const unsigned int AM_FREQ[];
extern const unsigned int FM_FREQ[];
extern const unsigned int HW_RegAM[];
extern const unsigned int HW_RegFM[];
extern const unsigned int HW_Reg[];
extern unsigned int nREG5;


// BK1088 initialize
void Bk1088_Init(void) {
	int i;
	i=0;
	while (HW_Reg[i] !=0xff) {
		Bk1088_Write(HW_Reg[i],HW_Reg[i+1]);
		i+=2;
	 }
	i=0;
	while (HW_RegAM[i] !=0xff) {
		Bk1088_Write(HW_RegAM[i],HW_RegAM[i+1]);
		i+=2;
	 }
	nREG5 = 0x1fdd;
}

//Display Set
void Set_Freq(unsigned int band,unsigned int freq)
{
	if (band ==0) {						//AM mode MW522-1710KHz
		Bk1088_Write(0x05,0x1FDD);
		nREG5 = 0x1fDD;
		Bk1088_Write(0x07,0x2901);		//AM mode
		Bk1088_Write(0x05,0x1FDD);
		Bk1088_Write(0x07,0x2901);		//AM mode
		Bk1088_Write(0x0003,((freq-522)/9));
		Bk1088_Write(0x0003,((freq-522)/9) | 0x8000);
		Set_Num(freq);   	// SŎgw
	}
	if (band ==1) {						//SW mode SW2.3-21.85MHz
		Bk1088_Write(0x05,0x1F1D);		// BUG FIX 20130113
		nREG5 = 0x1F1D;					// BUG FIX 20130113
		Bk1088_Write(0x07,0x2901);		// AM mode
		Bk1088_Write(0x05,0x1F1D);		// BUG FIX 20130113
		Bk1088_Write(0x07,0x2901);		// AM mode
		Bk1088_Write(0x0003,((freq-2300)/1));
		Bk1088_Write(0x0003,((freq-2300)/1) | 0x8000);
		Set_Num(freq);   	// SŎgw
	}
	if (band ==2) {						//FM mode 76-91MHz
		Bk1088_Write(0x05,0x375d);
		nREG5 = 0x375d;
		Bk1088_Write(0x07,0x0901);		//FM mode
		Bk1088_Write(0x05,0x375d);
		Bk1088_Write(0x07,0x0901);		//FM mode
		Bk1088_Write(0x0003,(freq-7600)/10);
		Bk1088_Write(0x0003,((freq-7600)/10) | 0x8000);
		Set_Num(freq/10);   	// SŎgw
	}

	Set_Band(band);		// ohw 0=AM / 1=SW / 2=FM

}

// BK1088 Register access
// Reg write
void Bk1088_Write(unsigned int ad,unsigned int dt) {
	I2c_Start();			// start bit
	I2c_Write(0x0080);		// Device ID
	I2c_Write(ad << 1);		// Register address
	I2c_Write(dt>>8);			// Register data H
	I2c_Write(dt & 0xff);		// Register data L
	PORTCbits.RC8  = 1;		// DATA =H
	Wait_Time(1);
	I2c_Stop();				// Stop bit	
}

// Reg Read
unsigned int Bk1088_Read(unsigned int ad) {
	unsigned int dtH,dtL;
	I2c_Start();			// start bit
	I2c_Write(0x0080);		// Device ID
	I2c_Write((ad << 1) | 0x01);		// Register address + Read flag
	dtH = I2c_Read();		// Register data read H
	I2c_Ack();				// ACK output
	dtL = I2c_Read();		// Register data read L
	I2c_Nack();				// NACK output
	I2c_Stop();				// Stop bit	
	return ((dtH <<8) + dtL);
}

// software I2C RC8=DATA, RC9=CLK
// I2C Start
void I2c_Start(void){
	TRISC	=	0x00ff;
	Wait_Time(1);
	PORTCbits.RC8  = 0;		// DATA =L
	Wait_Time(1);
	PORTCbits.RC9  = 0;		// DATA& CLK = L
	Wait_Time(1);
}
// I2C Stop
void I2c_Stop(void){
	TRISC	=	0x00ff;	// 
	Wait_Time(1);
	PORTCbits.RC9  = 1;		// CLK = H
	Wait_Time(1000);
}
// I2C 8bit write and ACK
void I2c_Write(unsigned int dt){
	unsigned int i,wk;
	wk =dt;
	TRISC	=	0x00ff;	// 
	Wait_Time(1);
	for (i=0;i<8;i++) {
		PORTCbits.RC9  = 0;		// CLK = L
		Wait_Time(1);
		if((wk & 0x80)==0){PORTCbits.RC8 = 0;}else{PORTCbits.RC8 =1;} // data bit output
		wk <<=1;
		Wait_Time(1);
		PORTCbits.RC9  = 1;		// CLK = H
		Wait_Time(1);
		PORTCbits.RC9  = 0;		// CLK = L	
		Wait_Time(1);	
	}
	TRISC	=	0x01ff;	// RC8 = input
	Wait_Time(1);
	PORTCbits.RC9  = 1;		// CLK = H	
	Wait_Time(1);
	PORTCbits.RC9  = 0;		// CLK = L	
	Wait_Time(1);
	TRISC	=	0x00ff;	// RC8 = output
}

// I2C 8bit read
unsigned int I2c_Read(void){
	unsigned int i,wk;
	wk =0;
	TRISC	=	0x01ff;	// DATA input
	for (i=0;i<8;i++) {
		wk <<=1;
		PORTCbits.RC9  = 0;		// CLK = L
		Wait_Time(1);
		PORTCbits.RC9  = 1;		// CLK = H	
		if(PORTCbits.RC8 == 1){wk |= 1;}  // data bit input
		Wait_Time(1);
		PORTCbits.RC9  = 0;		// CLK = L
	}	
	return(wk);
}
void I2c_Ack(void) {
	TRISC	=	0x00ff;	// DATA output
	PORTCbits.RC8  = 0;		// ACK output
	Wait_Time(1);
	PORTCbits.RC9  = 1;		// CLK = H
	Wait_Time(1);
	PORTCbits.RC9  = 0;		// CLK = L
	Wait_Time(1);
}

void I2c_Nack(void) {
	TRISC	=	0x00ff;	// DATA output
	PORTCbits.RC8  = 1;		// NACK output
	Wait_Time(1);
	PORTCbits.RC9  = 1;		// CLK = H
	Wait_Time(1);
	PORTCbits.RC9  = 0;		// CLK = L
	Wait_Time(1);
}